from scarfer.format.interface import FormatInterface
import os

class TextFormatter(FormatInterface):

    def _format_file(self, f, settings):
        ret = []
        
        copyrights = settings.get('copyrights')
        licenses = settings.get('licenses')
        matches = settings.get('matches')

        if licenses:
            ret.append("{path}: {license}".format(path=f['path'], license=str(f['license']['expressions'])))
        else:
            ret.append("{path}".format(path=f['path']))
                
        if copyrights:
            for cr in f['copyrights']:
                ret.append(" * {c}".format(c=cr))
        if matches:
            for m in f['license']['matches']:
                ret.append("--- {match} --- ".format(match=m['key']))
                ret.append("{text}".format(text=m['text']))
            
        return "\n".join(ret)

    def _format_files(self, report, settings):
        ret = []
        for f in report['files']:
            ret.append(self._format_file(f, settings))
        return "\n".join(ret)

    def format(self, report, settings):
        results = [
            "Files:\n----------------------------",
            self._format_files(report, settings),
        ]
        return "\n".join(results)

    def format_fixes(self, fixes, settings={}):
        excluded = [ 'Filtered out:' ]
        for f in fixes['excluded_files']:
            excluded.append(f' * {f["path"]}')
        missing = ['Missing license fixed:'] 
        for curation in fixes['missing_license']:
            missing.append(f' * {curation["file"]} -> {curation["curation"]}')
        return f'{os.linesep.join(excluded)}{os.linesep}{os.linesep.join(missing)}'

    def format_cumulative(self, report, settings={}):
        ret = []
        cumulative = report['cumulative']
        ret.append(f"Cumulative license: {cumulative['license']}")
        return "\n".join(ret)
        
    def format_license_summary(self, report, settings={}):
        license_summary = set()
        for f in report['files']:
            for l in f['license']['expressions']:
                license_summary.add(l)
        return f'License:\n { " AND ".join(license_summary)}\n'

    def format_copyright_summary(self, report, settings={}):
        copyright_summary = set()
        for f in report['files']:
            for l in f['copyrights']:
                copyright_summary.add(l)
        c_list = list(copyright_summary)
        c_list.sort()
        c_string = "\n".join(c_list)
        return f'Copyrights:\n{c_string}\n'



        
        



from scarfer.format.format_json import JsonFormatter
from scarfer.format.format_yaml import YamlFormatter
from scarfer.format.format_text import TextFormatter
from scarfer.format.format_markdown import MarkdownFormatter


class FormatFactory:

    @staticmethod
    def formatter(format):
        if format.lower() == "json":
            return JsonFormatter()
        elif format.lower() == "yaml" or format.lower() == "yaml":
            return YamlFormatter()
        elif format.lower() == "text" or format.lower() == "txt":
            return TextFormatter()
        elif format.lower() == "markdown" or format.lower() == "md":
            return MarkdownFormatter()



import re
from enum import Enum
from license_expression import Licensing, LicenseSymbol



class ScanReportFilterOperator(Enum):
    AND = 1
    OR = 2

    def __str__(self):
        return str(self.value)
    
class ScanReportFilterType(Enum):
    FILE = 1
    LICENSE = 2
    
    def __str__(self):
        return str(self.value)

class ScanReportFilter:

    def __init__(self, _expr, _type = ScanReportFilterType.FILE):
        self.expr = _expr
        self.type = _type

    def __str__(self):
        return "expr: {_expr}, type: {_type}, ".format(
            _expr = self.expr,
            _type = self.type)

class Analyzer:

    def __init__(self, report):
        self.normaliazed_report = report
        self.data = report['files']
        self.schema = None

    def _apply_filter_file(self, filt, f):
        if filt.type == ScanReportFilterType.FILE:
            ret = re.search(filt.expr, f['path'])
            return ret != None
        elif filt.type == ScanReportFilterType.LICENSE:
            for l in f['license']['expressions']:
                if re.search(filt.expr,l):
                    return True
            return False
        else:
            raise(ScanReportException("Unsupported filter type. This is weird."))
            

    def _apply_filters_file(self, f, filters, filter_on_files):
        
        if len(filters) == 0:
            return True
        
        ret  = False
        
        for filt in filters:
            if filter_on_files:
                if filt.type == ScanReportFilterType.FILE:
                    ret = ret or self._apply_filter_file(filt, f)
                    
            else:
                if filt.type == ScanReportFilterType.LICENSE:
                    ret = ret or self._apply_filter_file(filt, f)

        return ret

    def _generic_filter_count(self, filters, filter_type):
        count = 0
        for filt in filters:
            if filt.type == filter_type:
                count += 1
        return count
        
    def _file_filter_count(self, filters):
        return self._generic_filter_count(filters, ScanReportFilterType.FILE)
    
    def _license_filter_count(self, filters):
        return self._generic_filter_count(filters, ScanReportFilterType.LICENSE)

    def __licenses_simplified(self, files):
        licenses = set()
        for f in files:
            for lic in f['license']['expressions']:
                licenses.add(lic)

        licenses_with_parenthesises = []
        for l in licenses:
            licenses_with_parenthesises.append(f" ( {l} )")
        #print(f"\nlic par:     {licenses_with_parenthesises}")
        license_string = " and ".join(licenses_with_parenthesises)
        licensing = Licensing()
        parsed = licensing.parse(license_string)
        if parsed is None:
            simplified = parsed
        else:
            simplified = parsed.simplify()
        return simplified

    def curate_missing_license(self, curated_license):
        for f in self.data:
            if f['license']['expressions'] == [ 'missing' ]:
                f['license']['expressions'] = curated_license
                self.report_data['fixes']['missing_license'].append(
                    {
                        'file': f['path'],
                        'curation': curated_license
                    }
                )
        
        self.report_data['cumulative']['license'] = self.__licenses_simplified(self.report_data['files'])
    
    def curate_file_license(self, files, curated_license):
        for curated_file in files:
            for f in self.data:
                if curated_file in f['path']:
                    #print(f'------ file: {f["path"]}')
                    orig = f['license']['expressions']
                    f['license']['expressions'] = [ curated_license ]
                    self.report_data['fixes']['curated_licenses'].append(
                        {
                            'file': f['path'],
                            'curation': curated_license,
                            'original': orig
                        }
                    )
        
        self.report_data['cumulative']['license'] = self.__licenses_simplified(self.report_data['files'])
    
    def apply_filters(self, filters=[], exclude_filters=[], package=""):

        # filter in on files
        if self._file_filter_count(filters) > 0:
            keep_data = []
            discard_data = []
            for f in self.data:
                #print("f: " + str(f['path']))
                if not self._apply_filters_file(f, filters, True):
                    discard_data.append(f)
                else:
                    keep_data.append(f)
        else:
            keep_data = self.data
            discard_data = []
            
        # filter in on licenses
        if self._license_filter_count(filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, filters, False):
                    discard_data.append(f)
                else:
                    ret_data.append(f)
            keep_data = ret_data

        # filter out on files
        if self._file_filter_count(exclude_filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, exclude_filters, True):
                    ret_data.append(f)
                else:
                    discard_data.append(f)
            keep_data = ret_data
            
        # filter out on licenses
        if self._license_filter_count(exclude_filters) > 0:
            ret_data = []
            for f in keep_data:
                if not self._apply_filters_file(f, exclude_filters, False):
                    ret_data.append(f)
                else:
                    discard_data.append(f)

            keep_data = ret_data
                    

        for f in keep_data:
            # If no license identified, mark as missing
            if f['license']['expressions'] == []:
                f['license']['expressions'] = [ "missing" ]

        report_data = {}
        report_data['files'] = keep_data
        report_data['fixes'] = {}
        report_data['fixes']['excluded_files'] = discard_data
        report_data['fixes']['missing_license'] = []
        report_data['fixes']['curated_licenses'] = []
        report_data['meta'] = {}
        report_data['cumulative'] = {}
        report_data['cumulative']['license'] = self.__licenses_simplified(keep_data)
        self.report_data = report_data

    def report(self):
        return self.report_data
        
    def fixes(self):
        if self.report_data == None:
            self.apply_filters()
        return self.report_data['fixes']




class Settings:
    def __init__(self, copyrights = False, licenses = False, matches = False, cumulative = False, license_summary = False, copyright_summary = False):
        self.settings_map = {}
        self.settings_map['copyrights'] = copyrights
        self.settings_map['licenses'] = licenses
        self.settings_map['matches'] = matches
        self.settings_map['cumulative'] = cumulative
        self.settings_map['license_summary'] = license_summary
        self.settings_map['copyright_summary'] = copyright_summary

    def get(self, key):
        #print(" get " + key + " from " + str(self.settings_map) + " =====> " + str(self.settings_map.get(key, False)))
        return self.settings_map.get(key, False)

class FormatInterface:

    def __init__(self):
        return

    def format(self, report, settings={}):
        return 

    def format_fixes(self, report, settings={}):
        return 

    def format_license_summary(self, report, settings={}):
        return 

    def format_copyright_summary(self, report, settings={}):
        return

    def format_cumulative(self, report, settings={}):
        return 





#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2021 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import RawTextHelpFormatter
import argparse
import json
import sys
import os

from scarfer.format.factory import FormatFactory
from scarfer.scan_interface import ScanReportReader
from scarfer.scan_interface import ScanReportException
from scarfer.analyzer import Analyzer
from scarfer.format.interface import Settings
from scarfer.filter_utils import create_filters
from scarfer.config import scarfer_version
from scarfer.config import scarfer_name

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))

PROGRAM_NAME = scarfer_name
PROGRAM_DESCRIPTION = "Source code scan report file reporter reads scan reports and outputs selected information about filtered files"
PROGRAM_URL = "https://github.com/hesa/scarfer"
PROGRAM_EXAMPLES = "   Coming soon."
BUG_URL = "https://github.com/hesa/scarfer/issues"
PROGRAM_COPYRIGHT = "(c) 2021 Henrik Sandklef<hesa@sandklef.com>"
PROGRAM_LICENSE = "GPL-3.0-or-later"
PROGRAM_AUTHOR = "Henrik Sandklef"
PROGRAM_SUPPORTED_FORMATS = "   * Scancode Output Format version 1.0.0\n   * Scaancode Toolkit, version 21 and upwards\n"
PROGRAM_ATTRIBUTION = ""
PROGRAM_SEE_ALSO = ""

OUTPUT_FORMAT_JSON = "JSON"
OUTPUT_FORMAT_TEXT = "text"
OUTPUT_FORMAT_YAML = "text"
OUTPUT_FORMATS = [ OUTPUT_FORMAT_JSON, OUTPUT_FORMAT_TEXT, OUTPUT_FORMAT_YAML ]

DATE_FMT = '%Y-%m-%d'

def parse():

    description = "NAME\n  " + PROGRAM_NAME + "\n\n"
    description = description + "DESCRIPTION\n  " + PROGRAM_DESCRIPTION + "\n\n"
    
    epilog = ""
    epilog = epilog + "EXAMPLES\n\n" + PROGRAM_EXAMPLES + "\n\n"
    epilog = epilog + "SUPPORTED SCAN REPORT FORMATS\n" + PROGRAM_SUPPORTED_FORMATS + "\n\n"
    epilog = epilog + "AUTHOR\n  " + PROGRAM_AUTHOR + "\n\n"
    epilog = epilog + "REPORTING BUGS\n  File a ticket at " + PROGRAM_URL + "\n\n"
    epilog = epilog + "COPYRIGHT\n  Copyright " + PROGRAM_COPYRIGHT + ".\n  License " + PROGRAM_LICENSE + "\n\n"
    epilog = epilog + "SEE ALSO\n  " + PROGRAM_SEE_ALSO + "\n\n"
    
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=RawTextHelpFormatter
    )

    parser.add_argument('file',
                        type=str,
                        help='Scan report to use',
                        default=None)
    
    parser.add_argument('--normalize',
                        action='store_true',
                        help='quit after scan report normalization and outpout result',
                        default=False)
    
    parser.add_argument('-m', '--matched-text',
                            action='store_true',
                            help='output information about license matches',
                            default=False)
    
    parser.add_argument('-c', '--copyrights',
                            action='store_true',
                            help='output information about copyrights',
                            default=False)
    
    parser.add_argument('-cu', '--cumulative',
                            action='store_true',
                            help='outpur cumulative license information',
                            default=False)
    
    parser.add_argument('-l', '--license',
                            action='store_true',
                            help='output information about license',
                            default=False)
    
    parser.add_argument('-ls', '--license-summary',
                            action='store_true',
                            help='output license summary',
                            default=False)
    
    parser.add_argument('-cs', '--copyright-summary',
                            action='store_true',
                            help='output copyright summary',
                            default=False)
    
    parser.add_argument('-il', '--include-license',
                            type=str,
                            nargs="+",
                            action='append',
                            help='filter on licenses containing argument',
                            default=[])
    
    parser.add_argument('-el', '--exclude-license',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter out licenses containing argument',
                            default=[])
    
    parser.add_argument('-if', '--include-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter on file containing argument',
                            default=[])
    
    parser.add_argument('-ef', '--exclude-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter out on file containing argument',
                            default=[])
    
    parser.add_argument('-iff', '--include-file-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter on file containing file filters',
                            default=[])
    
    parser.add_argument('-eff', '--exclude-file-file',
                            type=str,
                            action='append',
                            nargs="+",
                            help='filter out on file containing file filters',
                            default=[])
    
    parser.add_argument('-cml', '--curate-missing-license',
                        type=str,
                        dest='curate_missing_license',
                        action='append',
                        help='curate missing license for a file with',
                        default=[])
    
    parser.add_argument('-cfl', '--curate-file-license',
                        type=str,
                        dest='curate_file_license',
                        nargs="+",
                        action='append',
                        help='curate license for a file with: file1 file2 curated-license',
                        default=[])
    
    parser.add_argument('-f', '--format',
                            type=str,
                            help='output result in specified format, default is ' + OUTPUT_FORMAT_TEXT,
                            default=OUTPUT_FORMAT_TEXT)

    parser.add_argument('--output-fixes',
                        dest='output_fixes',
                        action='store_true',
                        help='output files filtered out and curations')

    parser.add_argument('--version', '-V', action='version', version="{name}: {version}".format(name=scarfer_name, version=scarfer_version))

    parser.add_argument('-v', '--verbose',
                            action='store_true',
                            help='output verbose information to stderr',
                            default=False)
    
    parser.add_argument('-oc', '--output--config',
                        action='store_true',
                        dest="output_config",
                        help='output command line options as configuration',
                        default=False)
    
    parser.add_argument('--config',
                        type=str,
                        dest="read_config",
                        help='read configuration from file',
                        default=False)
    
    args = parser.parse_args()

    return args


def _merge_file_filters(clude_file, clude_file_file):
    new_filters = []

    for include_list in clude_file:
        for filter in include_list:
            new_filters.append(filter)

    for file_name_list in clude_file_file:
        for file_name in file_name_list:
            f = open(file_name, 'r')
            for line in f.readlines():
                stripped_line = line.strip().replace("\n","")
                if not stripped_line.startswith("#") and len(stripped_line) > 0:
                    reg_exp = stripped_line
                    if not stripped_line.endswith("/"):
                        reg_exp = stripped_line + "$"
                    new_filters.append(reg_exp)
 
    return new_filters

def flatten_lists(lists):
    new_list = []
    for l in lists:
        for elem in l:
            new_list.append(elem)
    return new_list

def _output_config(args):
    config = {
#        "copyrights": args.copyrights,
#        "cumulative": args.cumulative,
        "curate_missing_license": args.curate_missing_license,
        "curate_file_license": args.curate_file_license,
        "exclude_file": args.exclude_file,
        "exclude_file_file": args.exclude_file_file,
        "exclude_license": args.exclude_license,
        "file": args.file,
#        "format": args.format,
        "include_file": args.include_file,
        "include_file_file": args.include_file_file,
        "include_license": args.include_license,
#        "license": args.license,
#        "license_summary": args.license_summary,
#        "matched_text": args.matched_text
        }
    return json.dumps(config, indent=4)
        
def _read_config(config_file, args):
    new_args = {}
        
    for key in args.__dict__:
        new_args[key] = vars(args).get(key)

    if not args.read_config:
        return new_args
    
    with open(config_file) as conf:
        confs = json.load(conf)

    for key, value in confs.items():
        new_args[key] = value

    return new_args

    # 0xDEADC0DE
    for key in args.__dict__:
        args_val = vars(args)[key]
        conf_val = new_args.get(key)
        print(f'{key}: {args_val} : {conf_val}')
        if args_val == conf_val:
            #print(f'key {key} same')
            pass
        elif key not in args:
            print(f'key {key} not found')
            pass
        else:
            print(f'key {key} differs:  conf: {conf_val}')
            if isinstance(conf_val,list):
                new_args[key] += args_val
            else:
                new_args[key] = args_val
            print(f'key {key} now: {new_args[key]}')
        

def main():
    
    args = parse()

    if args.output_config:
        print(_output_config(args))
        sys.exit(0)
        
    args = _read_config(args.read_config, args)

    # Create scan report reader 
    reader = ScanReportReader(args['file'])

    # Get a normalized report
    normalized_report = reader.read()
    if args['normalize']:
        reader.validate()
        print(json.dumps(normalized_report, indent=4))
        sys.exit(0)
    
    include_license = flatten_lists(args['include_license'])
    exclude_license = flatten_lists(args['exclude_license'])
    #exclude_file = flatten_lists(args['exclude_file)
    
    # Create filters
    include_files = _merge_file_filters(args['include_file'], args['include_file_file'])
    filters = create_filters(include_license, include_files)
    
    exclude_files = _merge_file_filters(args['exclude_file'], args['exclude_file_file'])
    exclude_filters = create_filters(exclude_license, exclude_files)
    
#    print(f"include_files:   {include_files}")
#    print(f"include_filters: {filters}")
#    print(f"exclude_files:   {exclude_files}")
#    print(f"exclude_filters: {exclude_filters}")
    
    # Create output formatter
    formatter = FormatFactory.formatter(args['format'])

    # Create settings map, to pass to apply_filter
    settings = Settings(args['copyrights'], args['license'], args['matched_text'], args['cumulative'], args['license_summary'], args['copyright_summary'])

    # Filter the data in the report, with the filters
    analyzer = Analyzer(normalized_report)
    analyzer.apply_filters(filters, exclude_filters)

    #
    # Curations
    # 
    if args['curate_missing_license']:
        analyzer.curate_missing_license(args['curate_missing_license'])
    if args['curate_file_license']:
        for curation in args["curate_file_license"]:
            length = len(curation)
            nr_files = length -1
            curated_license = curation[nr_files]
            files = curation[0:nr_files]
            analyzer.curate_file_license(files, curated_license)
            
    filtered_files = analyzer.report()

    # Format the data
    if args['copyright_summary']:
        formatted_data = formatter.format_copyright_summary(filtered_files, settings)
    elif args['license_summary']:
        formatted_data = formatter.format_license_summary(filtered_files, settings)
    elif args['cumulative']:
        formatted_data = formatter.format_cumulative(filtered_files, settings)
    elif args['output_fixes']:
        formatted_data = formatter.format_fixes(reader.fixes(), settings)
    else:
        formatted_data = formatter.format(filtered_files, settings)

    # Print the data
    print(formatted_data)

if __name__ == '__main__':
    main()
    



import json

from scarfer.format.interface import FormatInterface

class JsonFormatter(FormatInterface):

    def format(self, report, settings={}):
        return json.dumps(report['files'], indent=4)

    def format_cumulative(self, report, settings={}):
        ret = []
        cumulative = report['cumulative']
        ret.append(" * license: {_license}".format(_license=cumulative['license']))
        return "\n".join(ret)
        
    def format_license_summary(self, report, settings={}):
        license_summary = set()
        for f in report['files']:
            for l in f['license']['expressions']:
                license_summary.add(l)
        return json.dumps({ "license": f'{ " AND ".join(license_summary)}' })

    def format_copyright_summary(self, report, settings={}):
        copyright_summary = set()
        for f in report['files']:
            for l in f['copyrights']:
                copyright_summary.add(l)
        c_list = list(copyright_summary)
        c_list.sort()
        c_string = "\n".join(c_list)
        return json.dumps({ "copyrights": f'\n{c_string}' })



        
        



import json
import jsonschema
import os

from scarfer.format.factory import FormatFactory
#from format import FormatInterface

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))


# expected result from read(), from the implementing classes
# ----------------------
# files: [
#   {
#     "path":       "tmp/tmp/apa.c",
#     "copyrights": [ "(c) 2009 Some One", "(c) 2001 Someone Elsie" ],
#     "license":    {
#       "expression": [ "mit or x11", "bsd" ],
#       "matches": [
#         {
#           "key":  "mit or x11",
#           "text": "This library is license under mit or x11"
#         }
#       ]
#     }
#  ]

#
# what to expect from report() - as a user
#
# {
#   "files": (see above)
#   "meta": {}
#   "cumulative": {
#     "license": "simplified license of all the combined"
#     "copytights": "combined copyrights"
#   }

            
class ScanReportReader:
    
    def __init__(self, file_name):
        self.file_name = file_name
        self.report_data = None
        self.schema = None

    def validate(self):
        if not self.schema:
            schema_file = os.path.join(os.path.join(SCRIPT_DIR, "var"), "normalized-scan.json")
            with open(schema_file, 'r') as f:
                self.schema = json.load(f)
            jsonschema.validate(instance=self.data, schema=self.schema)
        
    def read(self):
        clazzes = [ FakeReportReader, ScancodeReportReader ]
        self.data = None
        for clazz in clazzes:
            try:
                self.data = clazz(self.file_name).read()
                break
            except Exception as e:
                pass
        if self.data == None:
            raise(ScanReportException(f"File {self.file_name} not in a supported format"))
        
        return self.data

    def __str__(self):
        ret = ["path: {name}".format(name=self.file_name)]
        return '\n'.join(ret)
            
class ScanReportException(Exception):

    def __init__(self, message=""):
        self.message = message
        super().__init__(self.message)

class FakeReportReader(ScanReportReader):

    def read(self):
        raise(ScanReportException("File not in fake format"))
        
class ScancodeReportReader(ScanReportReader):

    def check_file_format(self):
        headers = self.json_data['headers'][0]
        tool = headers['tool_name']
        self.scancode_format = headers.get('output_format_version', '')
        if tool.lower() != "scancode-toolkit":
            raise(ScanReportException(f'File not in Scancode format. Tool={tool}'))

        if self.scancode_format == "2.0.0":
            self.copyright_value = "copyright"
        else:
            self.copyright_value = "value"
        self.tool = headers['tool_name']
        self.tool_version = headers['tool_version']
    
    def read(self):
        with open(self.file_name) as fp:
            import json
            self.json_data = json.load(fp)
            self.check_file_format()
            self.files = self.json_data['files']

        files = []


        for f in self.files:
            if not f['type'] == "file":
                continue
            
            _file = {}
            
            # path
            _file['path'] = f.get('path','')
            _file['sha1'] = f.get('sha1','')
            _file['md5'] = f.get('md5','')
            _file['sha256'] = f.get('sha256','')

            # copyright
            _file['copyrights'] = []
            for c in f['copyrights']:
                #print(" c: " + str(c) + "  reading: " + self.copyright_value)
                _file['copyrights'].append(c[self.copyright_value])
                pass

            # license
            _file['license'] = {}
            matches = []
            for l in f['licenses']:
                if 'key' in l and 'matched_text' in l:
                    matches.append({
                        "key": l['key'],
                        "text": l['matched_text']
                    })
            _file['license']['expressions'] = f['license_expressions']
            _file['license']['matches'] = matches
            
            files.append(_file)

        meta = {
            "scanner": {
                "tool_name": self.tool,
                "tool_version": self.tool_version,
                "tool_output_format": self.scancode_format
            }
        }

        return {
            "files": files,
            "meta": meta
        }




from scarfer.format.format_text import TextFormatter
#from scarfer.format.interface import FormatInterface
import os

class MarkdownFormatter(TextFormatter):

    def format_fixes(self, fixes, settings={}):
        excluded = [ 'Excluded files:\n' ]
        for f in fixes['excluded_files']:
            excluded.append(f' * {f["path"]}\n')
        missing = ['Missing license fixed:\n'] 
        for curation in fixes['missing_license']:
            missing.append(f' * {curation["file"]} -> {curation["curation"]}\n')
        return f'{os.linesep.join(excluded)}{os.linesep}{os.linesep.join(missing)}'

    def format_copyright_summary(self, report, settings={}):
        copyright_summary = set()
        for f in report['files']:
            for l in f['copyrights']:
                copyright_summary.add(f'{l}{os.linesep}')
        c_list = list(copyright_summary)
        c_list.sort()
        c_string = "\n".join(c_list)
        return f'{c_string}\n'

    



from scarfer.analyzer import ScanReportFilter
from scarfer.analyzer import ScanReportFilterType

def _create_filter(expr, type):
    return ScanReportFilter(expr, type)

def _create_license_filter(expr):
    return _create_filter(expr, ScanReportFilterType.LICENSE)
    
def _create_file_filter(expr):
    return _create_filter(expr, ScanReportFilterType.FILE)

def create_filters(license_filter, file_filter):
    filters = []
    for lf in license_filter:
        filters.append(_create_license_filter(lf))
        
    for ff in file_filter:
        filters.append(_create_file_filter(ff))
    return filters



# SPDX-FileCopyrightText: 2021 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import setuptools
from scarfer.config import scarfer_version

_long_description = "Scarfer outputs compliance related information from a scan report."


requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

requirements_dev = []
with open('requirements-dev.txt') as f:
    requirements_dev = f.read().splitlines()

setuptools.setup(
    name = "scarfer",
    version = scarfer_version,
    author = "Henrik Sanklef",
    author_email = "hesa@sandklef.com",
    description = "",
    long_description = _long_description,
    long_description_content_type = "text/markdown",
    license_files = ('LICENSES/GPL-3.0-or-later.txt',),
    url = "https://github.com/hesa/scarfer",
    packages = ['scarfer', 'scarfer/format'],
    entry_points = {
        "console_scripts": [
            "scarfer = scarfer.__main__:main",
        ],
    },
    package_data = {
        'scarfer': ['var/*'],
    },
    install_requires = requirements,
    extras_require = {
        'dev': requirements_dev,
    },
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Legal Industry",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Quality Assurance",
    ],
    python_requires = '>=3.6',
)
